/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


#ifndef __TOOLBOX_EXCEPTION_H
#define __TOOLBOX_EXCEPTION_H


/**
 * Creates a new EAssertionFailed together with trace information (file, line).
 * @param message The exception message.
 * @note Use this macro and do not instantiate directly.
 */
#define NEW_EASSERTIONFAILED(message) (new EAssertionFailed(message, __FILE__, __LINE__))

/**
 * Creates a new EDeviceError together with trace information (file, line).
 * @param message The exception message.
 * @note Use this macro and do not instantiate directly.
 */
#define NEW_EDEVICEERROR(message) (new EDeviceError(message, __FILE__, __LINE__))

#ifdef _WINDOWS
	/**
	* Creates a new EDeviceError together with trace information (file, line) and HRESULT.
	* @param message The exception message.
	* @param result The HRESULT value to be added to the message.
	* @note Use this macro and do not instantiate directly.
	*/
	#define NEW_EDEVICEERROR_RESULT(message, result) \
		(EDeviceError::CreateDeviceError((message), (result), __FILE__, __LINE__))
#endif

/**
 * Creates a new EIllegalArgument together with trace information (file, line).
 * @param message The exception message.
 * @note Use this macro and do not instantiate directly.
 */
#define NEW_EILLEGALARGUMENT(message) (new EIllegalArgument(message, __FILE__, __LINE__))

/**
 * Creates a new EIllegalState together with trace information (file, line).
 * @param message The exception message.
 * @note Use this macro and do not instantiate directly.
 */
#define NEW_EILLEGALSTATE(message) (new EIllegalState(message, __FILE__, __LINE__))

/**
 * Creates a new EIndexOutOfBounds together with trace information (file, line).
 * @param message The exception message.
 * @note Use this macro and do not instantiate directly.
 */
#define NEW_EINDEXOUTOFBOUNDS(message) (new EIndexOutOfBounds(message, __FILE__, __LINE__))

/**
 * Creates a new EInternalError together with trace information (file, line).
 * @param message The exception message.
 * @note Use this macro and do not instantiate directly.
 */
#define NEW_EINTERNALERROR(message) (new EInternalError(message, __FILE__, __LINE__))

/**
 * Creates a new EIOError together with trace information (file, line).
 * @param message The exception message.
 * @note Use this macro and do not instantiate directly.
 */
#define NEW_EIOERROR(message) (new EIOError(message, __FILE__, __LINE__))

/**
 * Creates a new ENoSuchElement together with trace information (file, line).
 * @param message The exception message.
 * @note Use this macro and do not instantiate directly.
 */
#define NEW_ENOSUCHELEMENT(message) (new ENoSuchElement(message, __FILE__, __LINE__))

/**
 * Creates a new ENotSupported together with trace information (file, line).
 * @param message The exception message.
 * @note Use this macro and do not instantiate directly.
 */
#define NEW_ENOTSUPPORTED(message) (new ENotSupported(message, __FILE__, __LINE__))

/**
 * Creates a new ENullPointer together with trace information (file, line).
 * @param message The exception message.
 * @note Use this macro and do not instantiate directly.
 */
#define NEW_ENULLPOINTER(message) (new ENullPointer(message, __FILE__, __LINE__))


#ifdef _DEBUG
	/**
	 * Asserts the given expression to be true. If
	 * it is false, an EAssertionFailed is thrown.
	 * @note Use this macro and not the EAssertionFailed::Assertion
	 * method directly, as it provides file and line number debug infos.
	 * @note This macro is only executed in the debug version.
	 * @param assertion The expression to be evaluated.
	 */
	#define ASSERTION(assertion) \
		(EAssertionFailed::Assertion((assertion), #assertion, __FILE__, __LINE__))
#else
	/**
	 * Does not do anything.
	 * @note This macro definition is used in the release version.
	 */
	#define ASSERTION(assertion) (((0)))
#endif

#ifdef _DEBUG
	/**
	 * Asserts that a given pointer is not NULL.
	 * If it is false, an ENullPointer is thrown.
	 * @note Use this macro and not the ENullPointer::Assertion
	 * method directly, as it provides file and line number debug infos.
	 * @note This macro is only executed in the debug version.
	 * @param pointer The pointer to be evaluated.
	 */
	#define ASSERTION_POINTER(pointer) \
		(ENullPointer::AssertionValid((pointer), #pointer##" is null", __FILE__, __LINE__))
#else
	/**
	 * Does not do anything.
	 * @note This macro definition is used in the release version.
	 */
	#define ASSERTION_POINTER(pointer) (((0)))
#endif

#ifdef _DEBUG
	/**
	 * Asserts that the current object is not NULL, so
	 * the method wasn't invoked on a null object.
	 * If it is false, an ENullPointer is thrown.
	 * @note Use this macro and not the ENullPointer::Assertion
	 * method directly, as it provides file and line number debug infos.
	 * @note This macro is only executed in the debug version.
	 */
	#define ASSERTION_THIS \
		(ENullPointer::AssertionValid((this), "this is null", __FILE__, __LINE__))
#else
	/**
	 * Does not do anything.
	 * @note This macro definition is used in the release version.
	 */
	#define ASSERTION_THIS (((0)))
#endif


namespace toolbox
{
	/**
	 * The root class for every exception. Every time such an exception
	 * is created, a log entry is also written to help trace bugs.
	 * Use the corresponding macro to create the exception object.
	 * Use the existing sub-classes or create new exception types.
	 * @warning windows.h defines the keywords GetClassName and GetMessage.
	 * In order to make this class work, windows.h must either be included
	 * before this declaration, or it must not be included at all.
	 * @author Thomas Jacob
	 */
	class Exception
	{
		protected:

			/**
			 * The name of the class actually instantiated.
			 */
			const char * ClassName;

			/**
			 * The line number of the exception creation.
			 * This parameter is usually determined by the exception macro.
			 */
			long LineNumber;

			/**
			 * The exception message text.
			 */
			char * Message;

			/**
			 * The source file name of the exception creation.
			 * This parameter is usually determined by the exception macro.
			 */
			char * Source;

			/**
			 * Creates a new Exception.
			 * @param className The name of the class actually instantiated.
			 * @param message The exception message.
			 * @param source The source file name of the exception creation.
			 * This parameter is usually determined by the exception macro.
			 * @param lineNumber The line number of the exception creation.
			 * This parameter is usually determined by the exception macro.
			 */
			Exception(const char * className, const char * message, const char * source, long lineNumber);

			/**
			 * Sets the exception text.
			 * @param message The new exception text.
			 */
			void SetMessage(const char * message);

			/**
			 * Sets the source file name.
			 * @param source The new source file name.
			 */
			void SetSource(const char * source);

		public:

			/**
			 * Destroys the Exception.
			 */
			virtual ~Exception();

			/**
			 * Returns the name of the class actually instantiated.
			 * @return The name of the class actually instantiated.
			 */
			inline const char * GetClassName();

			/**
			 * Returns the exception message.
			 * @return The exception message.
			 */
			inline const char * GetMessage();

			/**
			 * Returns the line number of the exception creation.
			 * @return The line number of the exception creation.
			 */
			inline long GetLineNumber();

			/**
			 * Returns the source file name of the exception creation.
			 * @return The source file name of the exception creation.
			 */
			inline const char * GetSource();
	};

	/**
	 * The EAutoBreaking is the base exception class for all exceptions
	 * that cause a debug break in the debug version of the application.
	 * Do not instantiate this class directly.
	 */
	class EAutoBreaking : public Exception
	{
		protected:

			/**
			 * Creates a new EAutoBreaking.
			 * @param className The name of the class actually instantiated.
			 * @param message The exception message.
			 * @param source The source file name of the exception creation.
			 * This parameter is usually determined by the exception macro.
			 * @param lineNumber The line number of the exception creation.
			 * This parameter is usually determined by the exception macro.
			 */
			EAutoBreaking(const char * className, const char * message, const char * source, long lineNumber);
	};

	/**
	 * The EAssertionFailed exception is thrown if there is
	 * an unexpected condition that cannot happen if the program
	 * was bug-free. It is normally thrown by the ASSERTION macro.
	 * EAssertionFailed exceptions are usually thrown in the
	 * debug version only.
	 * @author Thomas Jacob
	 */
	class EAssertionFailed : public EAutoBreaking
	{
		private:

			/**
			 * Creates a new EAssertionFailed.
			 * @param message The exception message.
			 * @param source The source file name of the exception creation.
			 * This parameter is usually determined by the exception macro.
			 * @param lineNumber The line number of the exception creation.
			 * This parameter is usually determined by the exception macro.
			 */
			EAssertionFailed(const char * message, const char * source, long lineNumber);

		public:

			/**
			 * Assertion for the ASSERTION macro. This method
			 * throws an EAssertionFailed exception, if the given
			 * expression isn't true. Do not call this method
			 * directly, but use the ASSERTION macro.
			 * @param assertion The expression to be evaluated.
			 * @param message The message to be displayed if the assertion fails.
			 * @param source The source file name of the ASSERTION macro call.
			 * @param lineNumber The line number of the ASSERTION macro call.
			 */
			static void Assertion(bool assertion, const char * message,
				const char * source, long lineNumber);

			/**
			 * Assertion for the ASSERTION macro. This method
			 * throws an EAssertionFailed exception, if the given
			 * expression isn't true. Do not call this method
			 * directly, but use the ASSERTION macro.
			 * @param assertion The expression to be evaluated.
			 * @param source The source file name of the ASSERTION macro call.
			 * @param lineNumber The line number of the ASSERTION macro call.
			 */
			static void Assertion(bool assertion, const char * source, long lineNumber);
	};

	/**
	 * The EDeviceError exception is thrown if there was
	 * an device error like a video card operation problem.
	 * @author Thomas Jacob
	 */
	class EDeviceError : public EAutoBreaking
	{
		public:

			/**
			 * Creates a new EDeviceError. Do not create this exception
			 * directly, but use the NEW_EDEVICEERROR(message) macro.
			 * @param message The exception message.
			 * @param source The source file name of the exception creation.
			 * This parameter is usually determined by the exception macro.
			 * @param lineNumber The line number of the exception creation.
			 * This parameter is usually determined by the exception macro.
			 */
			EDeviceError(const char * message, const char * source, long lineNumber);

			/**
			 * Creates a new EDeviceError. Do not create this exception
			 * directly, but use the NEW_EDEVICEERROR(message) macro.
			 * @param message The exception message.
			 * @param result The HRESULT value to be added to the message
			 * (e.g. as provided by GetLastError()).
			 * @param source The source file name of the exception creation.
			 * This parameter is usually determined by the exception macro.
			 * @param lineNumber The line number of the exception creation.
			 * This parameter is usually determined by the exception macro.
			 * @return The new EDeviceError.
			 */
			#ifdef _WINDOWS
				static EDeviceError * CreateDeviceError(const char * message,
					HRESULT result, const char * source, long lineNumber);

				/**
				 * Returns a description for an HRESULT error code (e.g. as provided by GetLastError()).
				 * @param result The HRESULT value to be added to the message
				 * (e.g. as provided by GetLastError()).
				 * @return The message.
				 */
				static String GetErrorMessage(HRESULT result);

				/**
				 * Returns a description for the last Win32 error occurred (as provided by GetLastError()).
				 * @return The message.
				 */
				static inline String GetLastErrorMessage();
			#endif
	};

	/**
	 * The EIllegalArgument exception is thrown if there was
	 * an illegal value, for example in the call to a method.
	 * @author Thomas Jacob
	 */
	class EIllegalArgument : public EAutoBreaking
	{
		public:

			/**
			 * Creates a new EIllegalArgument. Do not create this exception
			 * directly, but use the NEW_EILLEGALARGUMENT(message) macro.
			 * @param message The exception message.
			 * @param source The source file name of the exception creation.
			 * This parameter is usually determined by the exception macro.
			 * @param lineNumber The line number of the exception creation.
			 * This parameter is usually determined by the exception macro.
			 */
			EIllegalArgument(const char * message, const char * source, long lineNumber);
	};

	/**
	 * The EIllegalState exception is thrown if there was
	 * an illegal state, for example a method call before
	 * the object was prepared for that call.
	 * @author Thomas Jacob
	 */
	class EIllegalState : public EAutoBreaking
	{
		public:

			/**
			 * Creates a new EIllegalState. Do not create this exception
			 * directly, but use the NEW_EILLEGALSTATE(message) macro.
			 * @param message The exception message.
			 * @param source The source file name of the exception creation.
			 * This parameter is usually determined by the exception macro.
			 * @param lineNumber The line number of the exception creation.
			 * This parameter is usually determined by the exception macro.
			 */
			EIllegalState(const char * message, const char * source, long lineNumber);
	};

	/**
	 * The EIndexOutOfBounds exception is thrown if there was
	 * an index that is less or more than the valid bounds of
	 * the set, whose element the index points to.
	 * @author Thomas Jacob
	 */
	class EIndexOutOfBounds : public EAutoBreaking
	{
		public:

			/**
			 * Creates a new EIndexOutOfBounds. Do not create this exception
			 * directly, but use the NEW_EINDEXOUTOFBOUNDS(message) macro.
			 * @param message The exception message.
			 * @param source The source file name of the exception creation.
			 * This parameter is usually determined by the exception macro.
			 * @param lineNumber The line number of the exception creation.
			 * This parameter is usually determined by the exception macro.
			 */
			EIndexOutOfBounds(const char * message, const char * source, long lineNumber);
	};

	/**
	 * The EInternalError exception is thrown if there was
	 * in internal state that was unexpected in all cases, but is
	 * too severe not to be checked.
	 * @author Thomas Jacob
	 */
	class EInternalError : public EAutoBreaking
	{
		public:

			/**
			 * Creates a new EInternalError. Do not create this exception
			 * directly, but use the NEW_EINTERNALERROR(message) macro.
			 * @param message The exception message.
			 * @param source The source file name of the exception creation.
			 * This parameter is usually determined by the exception macro.
			 * @param lineNumber The line number of the exception creation.
			 * This parameter is usually determined by the exception macro.
			 */
			EInternalError(const char * message, const char * source, long lineNumber);
	};

	/**
	 * The EIOError exception is thrown if there was
	 * an input/output error like a read or write fault to the
	 * hard disk.
	 * @author Thomas Jacob
	 */
	class EIOError : public EAutoBreaking
	{
		public:

			/**
			 * Creates a new EIOError. Do not create this exception
			 * directly, but use the NEW_EIOERROR(message) macro.
			 * @param message The exception message.
			 * @param source The source file name of the exception creation.
			 * This parameter is usually determined by the exception macro.
			 * @param lineNumber The line number of the exception creation.
			 * This parameter is usually determined by the exception macro.
			 */
			EIOError(const char * message, const char * source, long lineNumber);
	};

	/**
	 * The ENoSuchElement exception is thrown if an element
	 * was not found within a searched set of elements.
	 * @author Thomas Jacob
	 */
	class ENoSuchElement : public EAutoBreaking
	{
		public:

			/**
			 * Creates a new ENoSuchElement. Do not create this exception
			 * directly, but use the NEW_ENOSUCHELEMENT(message) macro.
			 * @param message The exception message.
			 * @param source The source file name of the exception creation.
			 * This parameter is usually determined by the exception macro.
			 * @param lineNumber The line number of the exception creation.
			 * This parameter is usually determined by the exception macro.
			 */
			ENoSuchElement(const char * message, const char * source, long lineNumber);
	};

	/**
	 * The ENotSupported exception is thrown if a functionality
	 * was used that is not yet implemented, fully or partially.
	 * @author Thomas Jacob
	 */
	class ENotSupported : public EAutoBreaking
	{
		public:

			/**
			 * Creates a new ENotSupported. Do not use this method directly,
			 * but use the NEW_ENOTSUPPORTED(message) macro.
			 * @param message The exception message.
			 * @param source The source file name of the exception creation.
			 * This parameter is usually determined by the exception macro.
			 * @param lineNumber The line number of the exception creation.
			 * This parameter is usually determined by the exception macro.
			 */
			ENotSupported(const char * message, const char * source, long lineNumber);
	};

	/**
	 * The ENullPointer exception is thrown if something is done with
	 * a null object, such as a method invocation. For example, it can
	 * be thrown by the ASSERTION_THIS macro.
	 * @author Thomas Jacob
	 */
	class ENullPointer : public EAutoBreaking
	{
		private:

			/**
			 * Creates a new ENullPointer.
			 * @param message The exception message.
			 * @param source The source file name of the exception creation.
			 * This parameter is usually determined by the exception macro.
			 * @param lineNumber The line number of the exception creation.
			 * This parameter is usually determined by the exception macro.
			 */
			ENullPointer(const char * message, const char * source, long lineNumber);

		public:

			/**
			 * Assertion for the ASSERTION_THIS macro. This method
			 * throws an ENullPointer exception, if the given pointer
			 * is NULL. Do not call this method directly, but use
			 * the ASSERTION_THIS macro.
			 * @param pointer The pointer not to be NULL.
			 * @param message The message to be displayed if the assertion fails.
			 * @param source The source file name of the ASSERTION macro call.
			 * @param lineNumber The line number of the ASSERTION macro call.
			 */
			static void AssertionValid(const void * pointer, const char * message,
				const char * source, long lineNumber);

			/**
			 * Assertion for the ASSERTION_THIS macro. This method
			 * throws an ENullPointer exception, if the given pointer
			 * is NULL. Do not call this method directly, but use
			 * the ASSERTION_THIS macro.
			 * @param pointer The pointer not to be NULL.
			 * @param source The source file name of the ASSERTION macro call.
			 * @param lineNumber The line number of the ASSERTION macro call.
			 */
			static void AssertionValid(const void * pointer, const char * source, long lineNumber);
	};

	/**
	 * The EOutOfMemory exception is thrown if there was
	 * not enough free memory for a memory allocation.
	 * @author Thomas Jacob
	 */
	class EOutOfMemory : public Exception
	{
		private:

			/**
			 * The EOutOfMemory singleton. The EOutOfMemory exception is
			 * designed as a singleton, because when it is needed, there
			 * is no memory to create the exception object.
			 */
			static EOutOfMemory Singleton;

			/**
			 * Creates a new EOutOfMemory. It cannot be instantiates except
			 * once by the class itself, because when it is needed, there
			 * is no memory to create the exception object. Use the Get()
			 * method to get the exception.
			 */
			EOutOfMemory();

		public:

			/**
			 * Returns the single EOutOfMemory object. Use this method
			 * to receive the EOutOfMemory exception to be thrown if
			 * a memory allocation fails.
			 * @return The single EOutOfMemory object.
			 */
			static inline EOutOfMemory * Get();
	};
}


#endif
