/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


#ifndef __TOOLBOX_STRING_H
#define __TOOLBOX_STRING_H


// TODO: String-Ressourcen-Dateien
// TODO: Optional Lokalisierung
// TODO: Optional Unicode

/**
 * The number of bytes to be allocated if the text does not fit into the
 * buffer anymore. Allocations are also aligned to multiples of this value.
 */
#define _TB_ST_SIZEINCREMENT 16


// Helper defines for plain C style strings

#define CLONE_STRING(attribute, value) \
	{ \
		if ((attribute = new char[strlen(value) + 1]) == NULL) \
			throw EOutOfMemory::Get(); \
		strcpy(attribute, (value)); \
	}

#define UPDATE_STRING(attribute, value) \
	{ \
		if (attribute != NULL) {delete attribute;} \
		if ((attribute = new char[strlen(value) + 1]) == NULL) \
			throw EOutOfMemory::Get(); \
		strcpy(attribute, (value)); \
	}


namespace toolbox
{
	/**
	 * The string class represents an encapsulation of a character array,
	 * providing automatic memory management.
	 * It provides methods to concat, find, substring etc.
	 * The class always allocates some extra bytes so that many operations
	 * do not reallocate too often.
	 * @author Thomas Jacob
	 */
	#if defined(_DEBUG) && (defined(_AFX) || defined(_AFXDLL))
		class String : public CObject
	#else
		class String
	#endif
	{
		private:

			/**
			 * The buffer that carries the string data.
			 * The string is always 0 terminated.
			 * The size of the allocated memory is contained in the Size attribute.
			 * Because the buffer contains the 0 character,
			 * it is always larger than the string length.
			 */
			char * Data;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

			/**
			 * The size of the allocated buffer 'Data'.
			 */
			unsigned int Size;

			/**
			 * Allocates memory for the Data and sets into this variable.
			 * The old pointer is overwritten. The Size is also updated.
			 * The method ensures that a given amount of bytes is allocated
			 * at least, but the method allocates some more memory to reduce
			 * reallocation on string changes.
			 * @param minimumSize The minimum byte count to be allocated.
			 */
			void Allocate(unsigned int minimumSize);

		public:

			/**
			 * Creates a new empty string.
			 */
			String();

			/**
			 * Copy constructor: Creates a new string from another one.
			 * @param String The string that serves as a template.
			 */
			String(const class String & String);

			/**
			 * Creates a new string from another one.
			 * @param String The string that serves as a template.
			 */
			String(const class String * String);

			/**
			 * Creates a new string from another one.
			 * @param string The string that serves as a template.
			 */
			String(const char * string);

			/**
			 * Creates a new string from a substring of another string.
			 * @param string The string to read from.
			 * @param fromIndex The position of the first character to include.
			 * Must be valid.
			 * @param length The length of the substring to be extracted.
			 * Must be valid.
			 */
			String(const class String & String, unsigned int fromIndex, unsigned int length);

			/**
			 * Creates a new string from a substring of another string.
			 * @param string The string to read from.
			 * @param fromIndex The position of the first character to include.
			 * Must be valid.
			 * @param length The length of the substring to be extracted.
			 * Must be valid.
			 */
			String(const class String * String, unsigned int fromIndex, unsigned int length);

			/**
			 * Creates a new string from a substring of another string.
			 * @param string The string to read from.
			 * @param fromIndex The position of the first character to include.
			 * Must be valid.
			 * @param length The length of the substring to be extracted.
			 * Must be valid.
			 */
			String(const char * string, unsigned int fromIndex, unsigned int length);

			/**
			 * Creates a new string that contains one character.
			 * @param character The character to put into the string.
			 */
			String(char character);

			/**
			 * Creates a new string that contains an signed int number.
			 * @param number The long number to put into the string.
			 */
			String(int number);

			/**
			 * Creates a new string that contains an unsigned int number.
			 * @param number The unsigned long number to put into the string.
			 */
			inline String(unsigned int number);

			/**
			 * Creates a new string that contains an signed long number.
			 * @param number The long number to put into the string.
			 */
			String(long number);

			/**
			 * Creates a new string that contains an unsigned long number.
			 * @param number The unsigned long number to put into the string.
			 */
			String(unsigned long number);

			/**
			 * Creates a new string that contains 'true' or 'false'.
			 * @param boolean The boolean to put into the string.
			 */
			String(bool boolean);

			/**
			 * Destroys the string, also deleting the Data buffer.
			 * So the memory returned by the (const char *) operator
			 * must not be accessed anymore after the deletion.
			 */
			~String();

			/**
			 * Returns the concatenation of this string and another one.
			 * @note Creating a new string by this method usually is less efficient
			 * than modifying an existing one (using += or Append).
			 * @param String The string to be added.
			 * @return The concatenation.
			 */
			inline String operator +(const class String & String) const;

			/**
			 * Returns the concatenation of this string and another one.
			 * @note Creating a new string by this method usually is less efficient
			 * than modifying an existing one (using += or Append).
			 * @param String The string to be added.
			 * @return The concatenation.
			 */
			inline String operator +(const class String * String) const;

			/**
			 * Returns the concatenation of this string and another one.
			 * @note Creating a new string by this method usually is less efficient
			 * than modifying an existing one (using += or Append).
			 * @param string The string to be added.
			 * @return The concatenation.
			 */
			inline String operator +(const char * string) const;

			/**
			 * Returns the concatenation of this string and a character.
			 * @note Creating a new string by this method usually is less efficient
			 * than modifying an existing one (using += or Append).
			 * @param character The character to be added.
			 * @return The concatenation.
			 */
			inline String operator +(char character) const;

			/**
			 * Returns the concatenation of this string and a signed int number.
			 * @note Creating a new string by this method usually is less efficient
			 * than modifying an existing one (using += or Append).
			 * @param number The number to be added.
			 * @return The concatenation.
			 */
			inline String operator +(int number) const;

			/**
			 * Returns the concatenation of this string and an unsigned int number.
			 * @note Creating a new string by this method usually is less efficient
			 * than modifying an existing one (using += or Append).
			 * @param number The number to be added.
			 * @return The concatenation.
			 */
			inline String operator +(unsigned int number) const;

			/**
			 * Returns the concatenation of this string and a signed long number.
			 * @note Creating a new string by this method usually is less efficient
			 * than modifying an existing one (using += or Append).
			 * @param number The number to be added.
			 * @return The concatenation.
			 */
			inline String operator +(long number) const;

			/**
			 * Returns the concatenation of this string and an unsigned long number.
			 * @note Creating a new string by this method usually is less efficient
			 * than modifying an existing one (using += or Append).
			 * @param number The number to be added.
			 * @return The concatenation.
			 */
			inline String operator +(unsigned long number) const;

			/**
			 * Returns the concatenation of this string and a boolean string.
			 * @note Creating a new string by this method usually is less efficient
			 * than modifying an existing one (using += or Append).
			 * @param boolean The boolean to be added.
			 * @return The concatenation.
			 */
			inline String operator +(bool boolean) const;

			/**
			 * Adds another string at the end of this one.
			 * @param String The string to be added.
			 */
			inline void operator +=(const class String & String);

			/**
			 * Adds another string at the end of this one.
			 * @param String The string to be added.
			 */
			inline void operator +=(const class String * String);

			/**
			 * Adds another string at the end of this one.
			 * @param string The string to be added.
			 */
			inline void operator +=(const char * string);

			/**
			 * Adds a character at the end of this string.
			 * @param character The character to be added.
			 */
			inline void operator +=(char character);

			/**
			 * Adds a signed int number at the end of this string.
			 * @param number The number to be added.
			 */
			inline void operator +=(int number);

			/**
			 * Adds an unsigned int number at the end of this string.
			 * @param number The number to be added.
			 */
			inline void operator +=(unsigned int number);

			/**
			 * Adds a signed long number at the end of this string.
			 * @param number The number to be added.
			 */
			inline void operator +=(long number);

			/**
			 * Adds an unsigned long number at the end of this string.
			 * @param number The number to be added.
			 */
			inline void operator +=(unsigned long number);

			/**
			 * Adds a boolean string at the end of this string.
			 * @param boolean The boolean to be added.
			 */
			inline void operator +=(bool boolean);

			/**
			 * Sets another string into this one.
			 * @param String The string to be set.
			 */
			inline void operator =(const class String & String);

			/**
			 * Sets another string into this one.
			 * @param String The string to be set.
			 */
			inline void operator =(const class String * String);

			/**
			 * Sets another string into this one.
			 * @param string The string to be set.
			 */
			void operator =(const char * string);

			/**
			 * Sets a character into this string.
			 * @param character The character to be set.
			 */
			void operator =(char character);

			/**
			 * Sets a signed int number into this string.
			 * @param number The number to be set.
			 */
			void operator =(int number);

			/**
			 * Sets an unsigned int number into this string.
			 * @param number The number to be set.
			 */
			inline void operator =(unsigned int number);

			/**
			 * Sets a signed long number into this string.
			 * @param number The number to be set.
			 */
			void operator =(long number);

			/**
			 * Sets an unsigned long number into this string.
			 * @param number The number to be set.
			 */
			void operator =(unsigned long number);

			/**
			 * Sets a boolean string into this string.
			 * @param boolean The boolean to be set.
			 */
			void operator =(bool boolean);

			/**
			 * Returns whether this string is less than another one,
			 * in terms of the Compare function.
			 * @param String The other string.
			 * @return Whether this string is less than another one.
			 */
			inline bool operator <(const class String & String) const;

			/**
			 * Returns whether this string is less than another one,
			 * in terms of the Compare function.
			 * @param String The other string.
			 * @return Whether this string is less than another one.
			 */
			inline bool operator <(const class String * String) const;

			/**
			 * Returns whether this string is less than another one,
			 * in terms of the Compare function.
			 * @param string The other string.
			 * @return Whether this string is less than another one.
			 */
			inline bool operator <(const char * string) const;

			/**
			 * Returns whether this string is less than or equals another one,
			 * in terms of the Compare function.
			 * @param String The other string.
			 * @return Whether this string is less than or equals another one.
			 */
			inline bool operator <=(const class String & String) const;

			/**
			 * Returns whether this string is less than or equals another one,
			 * in terms of the Compare function.
			 * @param String The other string.
			 * @return Whether this string is less than or equals another one.
			 */
			inline bool operator <=(const class String * String) const;

			/**
			 * Returns whether this string is less than or equals another one,
			 * in terms of the Compare function.
			 * @param string The other string.
			 * @return Whether this string is less than or equals another one.
			 */
			inline bool operator <=(const char * string) const;

			/**
			 * Returns whether this string is greater than another one,
			 * in terms of the Compare function.
			 * @param String The other string.
			 * @return Whether this string is greater than another one.
			 */
			inline bool operator >(const class String & String) const;

			/**
			 * Returns whether this string is greater than another one,
			 * in terms of the Compare function.
			 * @param String The other string.
			 * @return Whether this string is greater than another one.
			 */
			inline bool operator >(const class String * String) const;

			/**
			 * Returns whether this string is greater than another one,
			 * in terms of the Compare function.
			 * @param string The other string.
			 * @return Whether this string is greater than another one.
			 */
			inline bool operator >(const char * string) const;

			/**
			 * Returns whether this string is greater than or equals another one,
			 * in terms of the Compare function.
			 * @param String The other string.
			 * @return Whether this string is greater than or equals another one.
			 */
			inline bool operator >=(const class String & String) const;

			/**
			 * Returns whether this string is greater than or equals another one,
			 * in terms of the Compare function.
			 * @param String The other string.
			 * @return Whether this string is greater than or equals another one.
			 */
			inline bool operator >=(const class String * String) const;

			/**
			 * Returns whether this string is greater than or equals another one,
			 * in terms of the Compare function.
			 * @param string The other string.
			 * @return Whether this string is greater than or equals another one.
			 */
			inline bool operator >=(const char * string) const;

			/**
			 * Returns whether this string equals another one,
			 * in terms of the Compare function.
			 * @note This operator is the same as the Equals method.
			 * @param String The other string.
			 * @return Whether this string equals another one.
			 * @see Equals()
			 * @see EqualsIC()
			 */
			inline bool operator ==(const class String & String) const;

			/**
			 * Returns whether this string equals another one,
			 * in terms of the Compare function.
			 * @note This operator is the same as the Equals method.
			 * @param String The other string.
			 * @return Whether this string equals another one.
			 * @see Equals()
			 * @see EqualsIC()
			 */
			inline bool operator ==(const class String * String) const;

			/**
			 * Returns whether this string equals another one,
			 * in terms of the Compare function.
			 * @note This operator is the same as the Equals method.
			 * @param string The other string.
			 * @return Whether this string equals another one.
			 * @see Equals()
			 * @see EqualsIC()
			 */
			inline bool operator ==(const char * string) const;

			/**
			 * Returns whether this string does not equal another one,
			 * in terms of the Compare function.
			 * @param String The other string.
			 * @return Whether this string does not equal another one.
			 * @see Equals()
			 * @see EqualsIC()
			 */
			inline bool operator !=(const class String & String) const;

			/**
			 * Returns whether this string does not equal another one,
			 * in terms of the Compare function.
			 * @param String The other string.
			 * @return Whether this string does not equal another one.
			 * @see Equals()
			 * @see EqualsIC()
			 */
			inline bool operator !=(const class String * String) const;

			/**
			 * Returns whether this string does not equal another one,
			 * in terms of the Compare function.
			 * @param string The other string.
			 * @return Whether this string does not equal another one.
			 * @see Equals()
			 * @see EqualsIC()
			 */
			inline bool operator !=(const char * string) const;

			/**
			 * Returns a character at a given index of the string.
			 * @param index The 0-indexed position of the character.
			 * It is legal for this parameter to be out of range.
			 * In this case, 0 is returned.
			 * @return The character of 0, if the index is out of range.
			 */
			inline char operator [](int index) const;

			/**
			 * Returns a C style string containing the data of this object.
			 * @return The C style string.
			 */
			inline operator const char *() const;

			/**
			 * Adds another string at the end of this one.
			 * @param String The string to be added.
			 */
			inline void Append(const class String & String);

			/**
			 * Adds another string at the end of this one.
			 * @param String The string to be added.
			 */
			inline void Append(const class String * String);

			/**
			 * Adds another string at the end of this one.
			 * @param string The string to be added.
			 */
			void Append(const char * string);

			/**
			 * Adds a character at the end of this string.
			 * @param character The character to be added.
			 */
			void Append(char character);

			/**
			 * Adds a signed int number at the end of this string.
			 * @param number The number to be added.
			 */
			void Append(int number);

			/**
			 * Adds an unsigned int number at the end of this string.
			 * @param number The number to be added.
			 */
			inline void Append(unsigned int number);

			/**
			 * Adds a signed long number at the end of this string.
			 * @param number The number to be added.
			 */
			void Append(long number);

			/**
			 * Adds an unsigned long number at the end of this string.
			 * @param number The number to be added.
			 */
			void Append(unsigned long number);

			/**
			 * Adds a boolean string at the end of this string.
			 * @param boolean The boolean to be added.
			 */
			void Append(bool boolean);

			/**
			 * Clears the string, removing the content and setting the length to zero.
			 */
			inline void Clear();

			/**
			 * Returns a stack memory clone of this string.
			 * @return The stack memory clone.
			 */
			inline String Clone() const;

			/**
			 * Compares two strings and returns -1, if the first is
			 * less than the second, 0, if both strings are equal and
			 * 1, if the first is greater than the second.
			 * @param String1 The first string to be compared.
			 * @param String2 The second string to be compared.
			 * @return -1, 0 or 1, if String1 &lt; String2, String1 == String2,
			 * resp. String1 &gt; String2.
			 */
			static inline int Compare(const String & String1, const String & String2);

			/**
			 * Compares two strings and returns -1, if the first is
			 * less than the second, 0, if both strings are equal and
			 * 1, if the first is greater than the second.
			 * @param String1 The first string to be compared.
			 * @param String2 The second string to be compared.
			 * @return -1, 0 or 1, if String1 &lt; String2, String1 == String2,
			 * resp. String1 &gt; String2.
			 */
			static inline int Compare(const String * String1, const String * String2);

			/**
			 * Compares two strings and returns -1, if the first is
			 * less than the second, 0, if both strings are equal and
			 * 1, if the first is greater than the second.
			 * @note This method is used for all case sensitive comparisons.
			 * @param string1 The first string to be compared.
			 * @param string2 The second string to be compared.
			 * @return -1, 0 or 1, if string1 &lt; string2, string1 == string2,
			 * resp. string1 &gt; string2.
			 */
			static int Compare(const char * string1, const char * string2);

			/**
			 * Compares two strings ignoring the case and returns -1, if the first is
			 * less than the second, 0, if both strings are equal and
			 * 1, if the first is greater than the second.
			 * @param String1 The first string to be compared.
			 * @param String2 The second string to be compared.
			 * @return -1, 0 or 1, if String1 &lt; String2, String1 == String2,
			 * resp. String1 &gt; String2 (ignoring the case).
			 */
			static inline int CompareIC(const String & String1, const String & String2);

			/**
			 * Compares two strings ignoring the case and returns -1, if the first is
			 * less than the second, 0, if both strings are equal and
			 * 1, if the first is greater than the second.
			 * @param String1 The first string to be compared.
			 * @param String2 The second string to be compared.
			 * @return -1, 0 or 1, if String1 &lt; String2, String1 == String2,
			 * resp. String1 &gt; String2 (ignoring the case).
			 */
			static inline int CompareIC(const String * String1, const String * String2);

			/**
			 * Compares two strings ignoring the case and returns -1, if the first is
			 * less than the second, 0, if both strings are equal and
			 * 1, if the first is greater than the second.
			 * @note This method is used for all case insensitive comparisons.
			 * @param string1 The first string to be compared.
			 * @param string2 The second string to be compared.
			 * @return -1, 0 or 1, if string1 &lt; string2, string1 == string2,
			 * resp. string1 &gt; string2 (ignoring the case).
			 */
			static int CompareIC(const char * string1, const char * string2);

			/**
			 * Compares this string to another one and returns -1, if this string is
			 * less than the given one, 0, if both strings are equal and
			 * 1, if this string is greater than the given one,
			 * in terms of the Compare function.
			 * @param String The string to compare to.
			 * @return -1, 0 or 1, if this &lt; String, this == String,
			 * resp. this &gt; String.
			 */
			inline int CompareTo(const String & String) const;

			/**
			 * Compares this string to another one and returns -1, if this string is
			 * less than the given one, 0, if both strings are equal and
			 * 1, if this string is greater than the given one,
			 * in terms of the Compare function.
			 * @param String The string to compare to.
			 * @return -1, 0 or 1, if this &lt; String, this == String,
			 * resp. this &gt; String.
			 */
			inline int CompareTo(const String * String) const;

			/**
			 * Compares this string to another one and returns -1, if this string is
			 * less than the given one, 0, if both strings are equal and
			 * 1, if this string is greater than the given one,
			 * in terms of the Compare function.
			 * @param string The string to compare to.
			 * @return -1, 0 or 1, if this &lt; string, this == string,
			 * resp. this &gt; string.
			 */
			inline int CompareTo(const char * string) const;

			/**
			 * Compares this string to another one ignoring the caseand returns -1,
			 * if this string is less than the given one, 0, if both strings are equal and
			 * 1, if this string is greater than the given one,
			 * in terms of the CompareIC function.
			 * @param String The string to compare to.
			 * @return -1, 0 or 1, if this &lt; String, this == String,
			 * resp. this &gt; String (ignoring the case).
			 */
			inline int CompareToIC(const String & String) const;

			/**
			 * Compares this string to another one ignoring the caseand returns -1,
			 * if this string is less than the given one, 0, if both strings are equal and
			 * 1, if this string is greater than the given one,
			 * in terms of the CompareIC function.
			 * @param String The string to compare to.
			 * @return -1, 0 or 1, if this &lt; String, this == String,
			 * resp. this &gt; String (ignoring the case).
			 */
			inline int CompareToIC(const String * String) const;

			/**
			 * Compares this string to another one ignoring the caseand returns -1,
			 * if this string is less than the given one, 0, if both strings are equal and
			 * 1, if this string is greater than the given one,
			 * in terms of the CompareIC function.
			 * @param string The string to compare to.
			 * @return -1, 0 or 1, if this &lt; string, this == string,
			 * resp. this &gt; string (ignoring the case).
			 */
			inline int CompareToIC(const char * string) const;

			/**
			 * Returns whether another string is contained in this string.
			 * @param String The string to be tested.
			 * @return Whether the given string is contained in this string.
			 */
			inline bool Contains(const class String & String) const;

			/**
			 * Returns whether another string is contained in this string.
			 * @param String The string to be tested.
			 * @return Whether the given string is contained in this string.
			 */
			inline bool Contains(const class String * String) const;

			/**
			 * Returns whether another string is contained in this string.
			 * @param string The string to be tested.
			 * @return Whether the given string is contained in this string.
			 */
			inline bool Contains(const char * string) const;

			/**
			 * Cuts off a part of the string, starting at a given index.
			 * The remaining string will contain the part before the index only.
			 * @param fromIndex The 0-indexed position of the first character to be cut.
			 * It is legal for this parameter to be out of range.
			 * In this case, the string no cut.
			 */
			void Cut(unsigned int fromIndex);

			/**
			 * Cuts off a part of the string, starting at a given index.
			 * The remaining string will contain the part before the index
			 * and the part after the index plus the provided cut length.
			 * @param fromIndex The 0-indexed position of the first character to be cut.
			 * It is legal for this parameter to be out of range.
			 * In this case, the string no cut.
			 * @param length The number of characters to be cut.
			 * It is legal for this parameter to be out of range.
			 * In this case, the whole right part is cut.
			 */
			void Cut(unsigned int fromIndex, unsigned int length);

			/**
			 * Returns whether another string is part of the end of this string.
			 * @param String The string to be tested.
			 * @return Whether the given string is part of the end of this string.
			 */
			inline bool EndsWith(const class String & String) const;

			/**
			 * Returns whether another string is part of the end of this string.
			 * @param String The string to be tested.
			 * @return Whether the given string is part of the end of this string.
			 */
			inline bool EndsWith(const class String * String) const;

			/**
			 * Returns whether another string is part of the end of this string.
			 * @param string The string to be tested.
			 * @return Whether the given string is part of the end of this string.
			 */
			bool EndsWith(const char * string) const;

			/**
			 * Returns whether another string is part of the end of this string,
			 * ignoring the case of the two strings.
			 * @param String The string to be tested.
			 * @return Whether the given string is part of the end of this string.
			 */
			inline bool EndsWithIC(const class String & String) const;

			/**
			 * Returns whether another string is part of the end of this string,
			 * ignoring the case of the two strings.
			 * @param String The string to be tested.
			 * @return Whether the given string is part of the end of this string.
			 */
			inline bool EndsWithIC(const class String * String) const;

			/**
			 * Returns whether another string is part of the end of this string,
			 * ignoring the case of the two strings.
			 * @param string The string to be tested.
			 * @return Whether the given string is part of the end of this string.
			 */
			bool EndsWithIC(const char * string) const;

			/**
			 * Returns whether this string equals another one,
			 * in terms of the Compare function.
			 * @note This method is the same as the '==' operator.
			 * @param String The other string.
			 * @return Whether this string equals another one.
			 */
			inline bool Equals(const class String & String) const;

			/**
			 * Returns whether this string equals another one,
			 * in terms of the Compare function.
			 * @note This method is the same as the '==' operator.
			 * @param String The other string.
			 * @return Whether this string equals another one.
			 */
			inline bool Equals(const class String * String) const;

			/**
			 * Returns whether this string equals another one,
			 * in terms of the Compare function.
			 * @note This method is the same as the '==' operator.
			 * @param string The other string.
			 * @return Whether this string equals another one.
			 */
			inline bool Equals(const char * string) const;

			/**
			 * Returns whether this string equals another one,
			 * in terms of the CompareIC function.
			 * @param String The other string.
			 * @return Whether this string equals another one.
			 */
			inline bool EqualsIC(const class String & String) const;

			/**
			 * Returns whether this string equals another one,
			 * in terms of the CompareIC function.
			 * @param String The other string.
			 * @return Whether this string equals another one.
			 */
			inline bool EqualsIC(const class String * String) const;

			/**
			 * Returns whether this string equals another one,
			 * in terms of the CompareIC function.
			 * @param string The other string.
			 * @return Whether this string equals another one.
			 */
			inline bool EqualsIC(const char * string) const;

			/**
			 * Returns the first position of another string in this one.
			 * @param String The string to be searched.
			 * @return The 0-indexed first position of the string or -1,
			 * if the string cannot be found.
			 */
			inline int Find(const class String & String) const;

			/**
			 * Returns the first position of another string in this one.
			 * @param String The string to be searched.
			 * @return The 0-indexed first position of the string or -1,
			 * if the string cannot be found.
			 */
			inline int Find(const class String * String) const;

			/**
			 * Returns the first position of another string in this one.
			 * @param string The string to be searched.
			 * @return The 0-indexed first position of the string or -1,
			 * if the string cannot be found.
			 */
			int Find(const char * string) const;

			/**
			 * Returns the first position of another string in this one,
			 * starting from a given position.
			 * @param String The string to be searched.
			 * @param fromIndex The 0-indexed position to start the search at.
			 * Only matches starting at fromIndex or higher are accounted for.
			 * It is legal for this parameter to be out of range.
			 * If the value is too large, the string is not searched.
			 * @return The 0-indexed first position of the string or -1,
			 * if the string cannot be found. The index counts from the
			 * start of the string, not from the given fromIndex.
			 * Only matches starting at fromIndex or higher are accounted for.
			 */
			inline int Find(const class String & String, unsigned int fromIndex) const;

			/**
			 * Returns the first position of another string in this one,
			 * starting from a given position.
			 * @param String The string to be searched.
			 * @param fromIndex The 0-indexed position to start the search at.
			 * Only matches starting at fromIndex or higher are accounted for.
			 * It is legal for this parameter to be out of range.
			 * If the value is too large, the string is not searched.
			 * @return The 0-indexed first position of the string or -1,
			 * if the string cannot be found. The index counts from the
			 * start of the string, not from the given fromIndex.
			 * Only matches starting at fromIndex or higher are accounted for.
			 */
			inline int Find(const class String * String, unsigned int fromIndex) const;

			/**
			 * Returns the first position of another string in this one,
			 * starting from a given position.
			 * @param string The string to be searched.
			 * @param fromIndex The 0-indexed position to start the search at.
			 * Only matches starting at fromIndex or higher are accounted for.
			 * It is legal for this parameter to be out of range.
			 * If the value is too large, the string is not searched.
			 * @return The 0-indexed first position of the string or -1,
			 * if the string cannot be found. The index counts from the
			 * start of the string, not from the given fromIndex.
			 * Only matches starting at fromIndex or higher are accounted for.
			 */
			int Find(const char * string, unsigned int fromIndex) const;

			/**
			 * Returns the first position of another string in this one, ignoring case.
			 * @param String The string to be searched.
			 * @return The 0-indexed first position of the string (ignoring case) or -1,
			 * if the string cannot be found.
			 */
			inline int FindIC(const class String & String) const;

			/**
			 * Returns the first position of another string in this one, ignoring case.
			 * @param String The string to be searched.
			 * @return The 0-indexed first position of the string (ignoring case) or -1,
			 * if the string cannot be found.
			 */
			inline int FindIC(const class String * String) const;

			/**
			 * Returns the first position of another string in this one, ignoring case.
			 * @param string The string to be searched.
			 * @return The 0-indexed first position of the string (ignoring case) or -1,
			 * if the string cannot be found.
			 */
			inline int FindIC(const char * string) const;

			/**
			 * Returns the first position of another string in this one, ignoring case,
			 * starting from a given position.
			 * @param String The string to be searched.
			 * @param fromIndex The 0-indexed position to start the search at.
			 * Only matches starting at fromIndex or higher are accounted for.
			 * It is legal for this parameter to be out of range.
			 * If the value is too large, the string is not searched.
			 * @return The 0-indexed first position of the string (ignoring case) or -1,
			 * if the string cannot be found. The index counts from the
			 * start of the string, not from the given fromIndex.
			 * Only matches starting at fromIndex or higher are accounted for.
			 */
			inline int FindIC(const class String & String, unsigned int fromIndex) const;

			/**
			 * Returns the first position of another string in this one, ignoring case,
			 * starting from a given position.
			 * @param String The string to be searched.
			 * @param fromIndex The 0-indexed position to start the search at.
			 * Only matches starting at fromIndex or higher are accounted for.
			 * It is legal for this parameter to be out of range.
			 * If the value is too large, the string is not searched.
			 * @return The 0-indexed first position of the string (ignoring case) or -1,
			 * if the string cannot be found. The index counts from the
			 * start of the string, not from the given fromIndex.
			 * Only matches starting at fromIndex or higher are accounted for.
			 */
			inline int FindIC(const class String * String, unsigned int fromIndex) const;

			/**
			 * Returns the first position of another string in this one, ignoring case,
			 * starting from a given position.
			 * @param string The string to be searched.
			 * @param fromIndex The 0-indexed position to start the search at.
			 * Only matches starting at fromIndex or higher are accounted for.
			 * It is legal for this parameter to be out of range.
			 * If the value is too large, the string is not searched.
			 * @return The 0-indexed first position of the string (ignoring case) or -1,
			 * if the string cannot be found. The index counts from the
			 * start of the string, not from the given fromIndex.
			 * Only matches starting at fromIndex or higher are accounted for.
			 */
			int FindIC(const char * string, unsigned int fromIndex) const;

			/**
			 * Returns the last position of another string in this one.
			 * @param String The string to be searched.
			 * @return The 0-indexed last position of the string or -1,
			 * if the string cannot be found.
			 */
			inline int FindLast(const class String & String) const;

			/**
			 * Returns the last position of another string in this one.
			 * @param String The string to be searched.
			 * @return The 0-indexed last position of the string or -1,
			 * if the string cannot be found.
			 */
			inline int FindLast(const class String * String) const;

			/**
			 * Returns the last position of another string in this one.
			 * @param string The string to be searched.
			 * @return The 0-indexed last position of the string or -1,
			 * if the string cannot be found.
			 */
			inline int FindLast(const char * string) const;

			/**
			 * Returns the last position of another string in this one,
			 * starting from a given position.
			 * @param String The string to be searched.
			 * @param fromIndex The 0-indexed position to start the search at.
			 * Only matches starting at fromIndex or less are accounted for.
			 * It is legal for this parameter to be out of range.
			 * If the value is too large, the string is searched from the end.
			 * @return The 0-indexed last position of the string or -1,
			 * if the string cannot be found. The index counts from the
			 * start of the string, not from the given fromIndex.
			 * Only matches starting at fromIndex or less are accounted for.
			 */
			inline int FindLast(const class String & String, unsigned int fromIndex) const;

			/**
			 * Returns the last position of another string in this one,
			 * starting from a given position.
			 * @param String The string to be searched.
			 * @param fromIndex The 0-indexed position to start the search at.
			 * Only matches starting at fromIndex or less are accounted for.
			 * It is legal for this parameter to be out of range.
			 * If the value is too large, the string is searched from the end.
			 * @return The 0-indexed last position of the string or -1,
			 * if the string cannot be found. The index counts from the
			 * start of the string, not from the given fromIndex.
			 * Only matches starting at fromIndex or less are accounted for.
			 */
			inline int FindLast(const class String * String, unsigned int fromIndex) const;

			/**
			 * Returns the last position of another string in this one,
			 * starting from a given position.
			 * @param string The string to be searched.
			 * @param fromIndex The 0-indexed position to start the search at.
			 * Only matches starting at fromIndex or less are accounted for.
			 * It is legal for this parameter to be out of range.
			 * If the value is too large, the string is searched from the end.
			 * @return The 0-indexed last position of the string or -1,
			 * if the string cannot be found. The index counts from the
			 * start of the string, not from the given fromIndex.
			 * Only matches starting at fromIndex or less are accounted for.
			 */
			int FindLast(const char * string, unsigned int fromIndex) const;

			/**
			 * Returns the last position of another string in this one, ignoring case.
			 * @param String The string to be searched.
			 * @return The 0-indexed last position of the string (ignoring case) or -1,
			 * if the string cannot be found.
			 */
			inline int FindLastIC(const class String & String) const;

			/**
			 * Returns the last position of another string in this one, ignoring case.
			 * @param String The string to be searched.
			 * @return The 0-indexed last position of the string (ignoring case) or -1,
			 * if the string cannot be found.
			 */
			inline int FindLastIC(const class String * String) const;

			/**
			 * Returns the last position of another string in this one, ignoring case.
			 * @param string The string to be searched.
			 * @return The 0-indexed last position of the string (ignoring case) or -1,
			 * if the string cannot be found.
			 */
			inline int FindLastIC(const char * string) const;

			/**
			 * Returns the last position of another string in this one, ignoring case,
			 * starting from a given position.
			 * @param String The string to be searched.
			 * @param fromIndex The 0-indexed position to start the search at.
			 * Only matches starting at fromIndex or less are accounted for.
			 * It is legal for this parameter to be out of range.
			 * If the value is too large, the string is searched from the end.
			 * @return The 0-indexed last position of the string (ignoring case) or -1,
			 * if the string cannot be found. The index counts from the
			 * start of the string, not from the given fromIndex.
			 * Only matches starting at fromIndex or less are accounted for.
			 */
			inline int FindLastIC(const class String & String, unsigned int fromIndex) const;

			/**
			 * Returns the last position of another string in this one, ignoring case,
			 * starting from a given position.
			 * @param String The string to be searched.
			 * @param fromIndex The 0-indexed position to start the search at.
			 * Only matches starting at fromIndex or less are accounted for.
			 * It is legal for this parameter to be out of range.
			 * If the value is too large, the string is searched from the end.
			 * @return The 0-indexed last position of the string (ignoring case) or -1,
			 * if the string cannot be found. The index counts from the
			 * start of the string, not from the given fromIndex.
			 * Only matches starting at fromIndex or less are accounted for.
			 */
			inline int FindLastIC(const class String * String, unsigned int fromIndex) const;

			/**
			 * Returns the last position of another string in this one, ignoring case,
			 * starting from a given position.
			 * @param string The string to be searched.
			 * @param fromIndex The 0-indexed position to start the search at.
			 * Only matches starting at fromIndex or less are accounted for.
			 * It is legal for this parameter to be out of range.
			 * If the value is too large, the string is searched from the end.
			 * @return The 0-indexed last position of the string (ignoring case) or -1,
			 * if the string cannot be found. The index counts from the
			 * start of the string, not from the given fromIndex.
			 * Only matches starting at fromIndex or less are accounted for.
			 */
			int FindLastIC(const char * string, unsigned int fromIndex) const;

			/**
			 * Formats a string the same way sprintf does.
			 * The result is put into this string.
			 * @param format The sprintf style format.
			 * @param ... The optional parameters, see sprintf for details.
			 */
			void Format(const char * format, ...);

			/**
			 * Formats a string the same way vsprintf does.
			 * The result is put into this string.
			 * @param format The sprintf style format.
			 * @param arguments The va_list arguments, see vsprintf for details.
			 */
			void FormatFromVargs(const char * format, const va_list & arguments);

			/**
			 * Returns the length of the string.
			 * @return The length.
			 */
			unsigned int GetLength() const;

			/**
			 * Returns a heap memory clone of this string.
			 * @return The heap memory clone.
			 */
			String * HeapClone() const;

			/**
			 * Inserts another string at a given position of this one.
			 * @param String The string to be added.
			 * @param index The 0-indexed position for the insertion.
			 * It is legal for this parameter to be out of range.
			 * In this case, the string is appended.
			 */
			inline void Insert(const class String & String, unsigned int index);

			/**
			 * Inserts another string at a given position of this one.
			 * @param String The string to be added.
			 * @param index The 0-indexed position for the insertion.
			 * It is legal for this parameter to be out of range.
			 * In this case, the string is appended.
			 */
			inline void Insert(const class String * String, unsigned int index);

			/**
			 * Inserts another string at a given position of this one.
			 * @param string The string to be added.
			 * @param index The 0-indexed position for the insertion.
			 * It is legal for this parameter to be out of range.
			 * In this case, the string is appended.
			 */
			void Insert(const char * string, unsigned int index);

			/**
			 * Inserts a character at a given position of this string.
			 * @param character The character to be added.
			 * @param index The 0-indexed position for the insertion.
			 * It is legal for this parameter to be out of range.
			 * In this case, the character is appended.
			 */
			void Insert(char character, unsigned int index);

			/**
			 * Inserts a signed int number at a given position of this string.
			 * @param number The number to be added.
			 * @param index The 0-indexed position for the insertion.
			 * It is legal for this parameter to be out of range.
			 * In this case, the number is appended.
			 */
			void Insert(int number, unsigned int index);

			/**
			 * Inserts an unsigned int number at a given position of this string.
			 * @param number The number to be added.
			 * @param index The 0-indexed position for the insertion.
			 * It is legal for this parameter to be out of range.
			 * In this case, the number is appended.
			 */
			inline void Insert(unsigned int number, unsigned int index);

			/**
			 * Inserts a signed long number at a given position of this string.
			 * @param number The number to be added.
			 * @param index The 0-indexed position for the insertion.
			 * It is legal for this parameter to be out of range.
			 * In this case, the number is appended.
			 */
			void Insert(long number, unsigned int index);

			/**
			 * Inserts an unsigned long number at a given position of this string.
			 * @param number The number to be added.
			 * @param index The 0-indexed position for the insertion.
			 * It is legal for this parameter to be out of range.
			 * In this case, the number is appended.
			 */
			void Insert(unsigned long number, unsigned int index);

			/**
			 * Inserts a boolean string at a given position of this string.
			 * @param boolean The boolean to be added.
			 * @param index The 0-indexed position for the insertion.
			 * It is legal for this parameter to be out of range.
			 * In this case, the boolean is appended.
			 */
			void Insert(bool boolean, unsigned int index);

			/**
			 * Returns whether a given character is a letter in the ISO-8859-1 encoding.
			 * @param character The character to be tested.
			 * @return Whether the character is a letter in the ISO-8859-1 encoding.
			 */
			static bool IsCharacter(unsigned char character);

			/**
			 * Returns whether a given character is a digit in the ISO-8859-1 encoding.
			 * @param character The character to be tested.
			 * @return Whether the character is a digit in the ISO-8859-1 encoding.
			 */
			static bool IsDigit(unsigned char character);

			/**
			 * Returns whether this string is empty (zero length).
			 * @return Whether this string is empty.
			 */
			inline bool IsEmpty() const;

			/**
			 * Returns whether a given character is a lower-case letter in the ISO-8859-1 encoding.
			 * @param character The character to be tested.
			 * @return Whether the character is a lower-case letter in the ISO-8859-1 encoding.
			 */
			static bool IsLowerCharacter(unsigned char character);

			/**
			 * Returns whether a given character is a upper-case letter in the ISO-8859-1 encoding.
			 * @param character The character to be tested.
			 * @return Whether the character is a upper-case letter in the ISO-8859-1 encoding.
			 */
			static bool IsUpperCharacter(unsigned char character);

			/**
			 * Returns whether a given character is part of the ISO-8859-1 encoding.
			 * @param character The character to be tested.
			 * @return Whether the character is part of the ISO-8859-1 encoding.
			 */
			static bool IsValid(unsigned char character);

			/**
			 * Returns whether a given character is white space in the ISO-8859-1 encoding.
			 * @param character The character to be tested.
			 * @return Whether the character is white space in the ISO-8859-1 encoding.
			 */
			static bool IsWhiteSpace(unsigned char character);

			/**
			 * Prepends another string at the beginning of this one.
			 * @param String The string to be prepended.
			 */
			inline void Prepend(const class String & String);

			/**
			 * Prepends another string at the beginning of this one.
			 * @param String The string to be prepended.
			 */
			inline void Prepend(const class String * String);

			/**
			 * Prepends another string at the beginning of this one.
			 * @param string The string to be prepended.
			 */
			void Prepend(const char * string);

			/**
			 * Prepends a character at the beginning of this string.
			 * @param character The character to be prepended.
			 */
			void Prepend(char character);

			/**
			 * Prepends a signed int number at the beginning of this string.
			 * @param number The number to be prepended.
			 */
			void Prepend(int number);

			/**
			 * Prepends an unsigned int number at the beginning of this string.
			 * @param number The number to be prepended.
			 */
			inline void Prepend(unsigned int number);

			/**
			 * Prepends a signed long number at the beginning of this string.
			 * @param number The number to be prepended.
			 */
			void Prepend(long number);

			/**
			 * Prepends an unsigned long number at the beginning of this string.
			 * @param number The number to be prepended.
			 */
			void Prepend(unsigned long number);

			/**
			 * Prepends a boolean string at the beginning of this string.
			 * @param boolean The boolean to be prepended.
			 */
			void Prepend(bool boolean);

			/**
			 * Reads the contents of a given file into this String, using a buffer.
			 * The previous contents of this String object is cleared in advance.
			 * @param fileName The name of the file to read from.
			 * This method internally opens and closes a FILE object.
			 * @param textMode Whether to translate CR and LF according to the platform
			 * the program is executed on. false disables any translation.
			 * @return Whether the file could be read. If this method returns false,
			 * the String is empty afterwards.
			 */
			bool ReadFrom(const char * fileName, bool textMode);

			/**
			 * Reads the contents of a given file into this String, using a buffer.
			 * The previous contents of this String object is cleared in advance.
			 * @param file The file to read from. It will not be closed by this method.
			 * @return Whether the file could be read. If this method returns false,
			 * the String is empty afterwards.
			 */
			bool ReadFrom(FILE * file);

			/**
			 * Finds all occurences of a string in this one
			 * and replaces it by another one.
			 * @param Text The text to be searched.
			 * @param Replacement The replacement for the matches.
			 */
			inline void Replace(const String & Text, const String & Replacement);

			/**
			 * Finds all occurences of a string in this one
			 * and replaces it by another one.
			 * @param Text The text to be searched.
			 * @param Replacement The replacement for the matches.
			 */
			inline void Replace(const String * Text, const String * Replacement);

			/**
			 * Finds all occurences of a string in this one
			 * and replaces it by another one.
			 * @param text The text to be searched.
			 * @param replacement The replacement for the matches.
			 */
			void Replace(const char * text, const char * replacement);

			/**
			 * Finds all occurences of a character in this string
			 * and replaces it by another one.
			 * @param character The character to be searched.
			 * @param replacement The replacement for the matches.
			 */
			void Replace(char character, char replacement);

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif

			/**
			 * Returns whether another string is part of the start of this string.
			 * @param String The string to be tested.
			 * @return Whether the given string is part of the start of this string.
			 */
			inline bool StartsWith(const class String & String) const;

			/**
			 * Returns whether another string is part of the start of this string.
			 * @param String The string to be tested.
			 * @return Whether the given string is part of the start of this string.
			 */
			inline bool StartsWith(const class String * String) const;

			/**
			 * Returns whether another string is part of the start of this string.
			 * @param string The string to be tested.
			 * @return Whether the given string is part of the start of this string.
			 */
			bool StartsWith(const char * string) const;

			/**
			 * Returns whether another string is part of the start of this string,
			 * ignoring the case of the two strings.
			 * @param String The string to be tested.
			 * @return Whether the given string is part of the start of this string.
			 */
			inline bool StartsWithIC(const class String & String) const;

			/**
			 * Returns whether another string is part of the start of this string,
			 * ignoring the case of the two strings.
			 * @param String The string to be tested.
			 * @return Whether the given string is part of the start of this string.
			 */
			inline bool StartsWithIC(const class String * String) const;

			/**
			 * Returns whether another string is part of the start of this string,
			 * ignoring the case of the two strings.
			 * @param string The string to be tested.
			 * @return Whether the given string is part of the start of this string.
			 */
			bool StartsWithIC(const char * string) const;

			/**
			 * Returns a substring of this one.
			 * @param fromIndex The position of the first character to include.
			 * It is legal for this parameter to be out of range.
			 * In this case, an empty string is returned.
			 * @param length The length of the substring to be extracted.
			 * It is legal for this parameter to be out of range.
			 * In this case, an empty string is returned.
			 * @return The substring.
			 */
			String Substring(unsigned int fromIndex, unsigned int length) const;

			/**
			 * Returns a substring of this one.
			 * @param fromIndex The position of the first character to include.
			 * It is legal for this parameter to be out of range.
			 * In this case, an empty string is returned.
			 * @return The substring starting from the index up to the end of the string.
			 */
			String Substring(unsigned int fromIndex) const;

			/**
			 * Converts all upper case characters of this string to lower case.
			 */
			void ToLowerCase();

			/**
			 * Converts a given character to lower case, if it is an upper case character.
			 * All other characters are kept untouched.
			 * @param character The character to be converted.
			 * @return The converted character.
			 */
			static unsigned char ToLowerCase(unsigned char character);

			/**
			 * Converts all lower case characters of this string to upper case.
			 */
			void ToUpperCase();

			/**
			 * Converts a given character to upper case, if it is an lower case character.
			 * All other characters are kept untouched.
			 * @param character The character to be converted.
			 * @return The converted character.
			 */
			static unsigned char ToUpperCase(unsigned char character);

			/**
			 * Removes all white space (space, CR, LF, FF, and tab)
			 * from the start and the end of this string, but not in the middle.
			 */
			void Trim();

			/**
			 * Removes all white space (space, CR, LF, FF, and tab)
			 * from the start of this string.
			 */
			void TrimLeft();

			/**
			 * Removes all white space (space, CR, LF, FF, and tab)
			 * from the end of this string.
			 */
			void TrimRight();

			/**
			 * Writes the contents of this String into a given file.
			 * @param fileName The name of the file to write to.
			 * This method internally opens and closes a FILE object.
			 * @param textMode Whether to translate CR and LF according to the platform
			 * the program is executed on. false disables any translation.
			 * @param append Whether to append to any existing file.
			 * If true, the String's contents is appended to the existing file's contents.
			 * If false, any existing file is overwritten with the String's contents.
			 * In either case, if the file does not exist, it is created.
			 * @return Whether the file could be written.
			 */
			bool WriteTo(const char * fileName, bool textMode, bool append = false);

			/**
			 * Writes the contents of this String into a given file.
			 * @param file The file to write to. It will not be closed by this method.
			 * @return Whether the file could be written.
			 */
			inline bool WriteTo(FILE * file);
	};
}


#endif
